{-# LANGUAGE FlexibleInstances, OverloadedStrings, NoImplicitPrelude, PackageImports #-}
module Main where

import Control.Applicative ((<$>))
import Control.Exception (bracket)
import Control.Monad
import Control.Monad.Trans
import Data.Acid
import Data.Acid.Local
import Data.Acid.Advanced
import Data.Aeson
import Happstack.Fay
import Happstack.Server
import MasterMind.Shared.Core
import MasterMind.Server.Play
import MasterMind.Server.Acid
import Fay.Convert
import "base" Prelude

-- NOTE: this file is autogenerated by running 'cabal build'. To load
-- this file into ghci you must first run 'cabal build' and pass
-- '-idist/build/autogen' to ghci. This directory contains a .ghci
-- file which will attempt to do the latter may not always working
-- depending on exactly how ghci is started.
--
-- Sorry about the hassle. There is a project to integrate cabal+ghci
-- which will make this less annoying in the future.
import Paths_mastermind

main :: IO ()
main =
    do dataFp         <- getDataFileName "data"
       mastermindHtml <- getDataFileName "data/mastermind.html"
       homeHtml       <- getDataFileName "data/home.html"
       bracket (openLocalStateFrom "_mastermind" initialGames) createCheckpointAndClose $ \acid ->
        simpleHTTP nullConf $
          do decodeBody (defaultBodyPolicy "/tmp/" 10000 10000 10000)
             msum $ [ dir "data"     $ serveDirectory DisableBrowsing [] dataFp
                    , dir "play"     $ serveFile (asContentType "text/html") mastermindHtml
                    , nullDir       >> serveFile (asContentType "text/html") homeHtml
                    , dir "ajax"     $ handleCommand (commandR acid)
                    , dir "new-game" $ newGameR acid

                    ]

-- | create a new game
newGameR :: AcidState Games
         -> ServerPart Response
newGameR acid =
    do method POST
       guess  <- liftIO newGuess
       gameId <- update' acid (NewGame guess)
       addCookie Session (mkCookie "gameid" (show $ unGameId gameId))
       seeOther ("/play" :: String) (toResponse ())

-- | handle an AJAX request
commandR :: AcidState Games
         -> Command
         -> ServerPart Response
commandR acid cmd =
    case cmd of
      (SendGuess guess rt) -> fayResponse rt $ sendGuessC acid guess
      (FetchBoard rt)      -> fayResponse rt $ fetchBoardC acid

-- | handle the 'SendCommand' AJAX request
sendGuessC :: AcidState Games
           -> Guess
           -> ServerPart (Maybe Row)
sendGuessC acid guess =
    do gid <- GameId <$> readCookieValue "gameid"
       mRow <- update' acid (AddGuess gid guess)
       return mRow

-- | handle the 'FetchBoard' AJAX request
fetchBoardC :: AcidState Games
            -> ServerPart (Maybe Board)
fetchBoardC acid =
    do gid <- GameId <$> readCookieValue "gameid"
       query' acid (GetBoard gid)
