{-# LANGUAGE DeriveDataTypeable, StandaloneDeriving, TemplateHaskell, TypeFamilies, RecordWildCards #-}
module MasterMind.Server.Acid where

import Control.Applicative
import Control.Monad.Reader
import Control.Monad.State
import Data.Acid
import Data.Data
import Data.SafeCopy
import Data.Map (Map)
import qualified Data.Map as Map
import MasterMind.Shared.Core
import MasterMind.Server.Play

$(deriveSafeCopy 0 'base ''Color)
$(deriveSafeCopy 0 'base ''Guess)
$(deriveSafeCopy 0 'base ''ScorePeg)
$(deriveSafeCopy 0 'base ''Score)
$(deriveSafeCopy 0 'base ''Row)
$(deriveSafeCopy 0 'base ''GameId)
$(deriveSafeCopy 0 'base ''Game)
$(deriveSafeCopy 0 'base ''Board)
$(deriveSafeCopy 0 'base ''Status)

deriving instance Ord GameId

data Games = Games
    { games      :: Map GameId Game
    , nextGameId :: GameId
    }
    deriving (Data, Typeable)
$(deriveSafeCopy 0 'base ''Games)

initialGames :: Games
initialGames = Games
    { games      = Map.empty
    , nextGameId = GameId 1
    }

addGuess :: GameId
         -> Guess
         -> Update Games (Maybe Row)
addGuess gid userGuess =
    do g@Games{..} <-get
       case Map.lookup gid games of
         Nothing -> return Nothing
         (Just game@Game{..}) ->
             do let row = Row userGuess (scoreGuess correctGuess userGuess)
                put $ g { games = Map.insert gid (game { board = Board (row : (rows board)) }) games }
                return (Just row)

newGame :: Guess -- ^ correct guess for this game
        -> Update Games GameId
newGame correctGuess =
    do g@Games{..} <- get
       let game = Game { correctGuess = correctGuess
                       , status       = Undecided
                       , board        = Board []
                       }
       put $ g { games      = Map.insert nextGameId game games
               , nextGameId = GameId $ (unGameId nextGameId) + 1
               }
       return nextGameId

getSolution :: GameId
            -> Query Games (Maybe Guess)
getSolution gid =
    do g@Games{..} <- ask
       return $ correctGuess <$> Map.lookup gid games
{-
addGuess :: GameId
         -> Guess
         -> Update Games (Maybe Row)
addGuess gid guess =
    do g@Games{..} <- get
       case Map.lookup gid games of
         Nothing -> return Nothing
         (Just game) ->
             do let row = Row guess (scoreGuess solution guess)
                put $ g { games = Map.insert gid (game { board = row : board }) }
-}
getBoard :: GameId
         -> Query Games (Maybe Board)
getBoard bid =
    do g@Games{..} <- ask
       return $ board <$> (Map.lookup bid games)

$(makeAcidic ''Games
  [ 'addGuess
  , 'newGame
  , 'getSolution
  , 'getBoard
  ])

