{-# LANGUAGE StandaloneDeriving #-}
module MasterMind.Server.Play where

import Data.List
import MasterMind.Shared.Core
import System.Random (Random(..))

scoreGuess :: Guess -- ^ correct pattern
           -> Guess -- ^ pattern to score
           -> Score
scoreGuess (Guess c1 c2 c3 c4) (Guess u1 u2 u3 u4) =
    let key = (cloc c1 u1) + (cloc c2 u2) + (cloc c3 u3) + (cloc c4 u4)
        clrs = 4 - length ([c1,c2,c3,c4] \\ [u1,u2,u3,u4])
        scorePeg n
            | key  >= n = Black
            | clrs >= n = White
            | otherwise = None
    in Score (scorePeg 1) (scorePeg 2) (scorePeg 3) (scorePeg 4)
    where
      cloc x y
          | x == y    = 1
          | otherwise = 0


deriving instance Bounded Color
deriving instance Enum Color

instance Random Color where
    randomR (a, b) g =
        case randomR (fromEnum a, fromEnum b) g of
          (x, g) -> (toEnum x, g)
    random g = randomR (minBound, maxBound) g

newGuess :: IO Guess
newGuess =
    do peg1 <- randomIO
       peg2 <- randomIO
       peg3 <- randomIO
       peg4 <- randomIO
       return (Guess peg1 peg2 peg3 peg4)
